!---------------------------------------------------------------------------------------
! FLEXINVERT: get_initfpctm
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> get_initfpctm
!!
!! Purpose:    Reads the initial concentration files and interpolates these to
!!             the flexpart grid
!!
!! Interface:
!!
!!    Inputs
!!             filename  -  name of 4D concentration file
!!             varname   -  variable name
!!             concini   -  concentrations on flexpart grid (zero array on input)
!! 
!!    Externals
!!             convertgrid
!!             interpz
!!             check
!!             
!---------------------------------------------------------------------------------------

subroutine get_initfpctm(filename, varname, concini)

  use mod_var
  use mod_save,    only : check
  use netcdf
  
  implicit none

  character(len=max_path_len),           intent(in)     :: filename
  character(len=max_name_len),           intent(in)     :: varname
  real, dimension(nxgrid,nygrid,nzgrid), intent(in out) :: concini

  character(len=*), parameter                           :: lonname='longitude'
  character(len=*), parameter                           :: latname='latitude'
  character(len=*), parameter                           :: prsname='pressure'
  character(len=*), parameter                           :: levname='lev'
  character(len=*), parameter                           :: timename='time'
  integer                                               :: ncid, xid, yid, zid, tid, varid
  integer                                               :: kz,jy, ix
  integer                                               :: nx, ny, nz, nt 
  integer                                               :: nx_out, ny_out
  logical                                               :: midpoints
  real, dimension(:), allocatable                       :: lon, lat, prs, geoh
  real, dimension(nxgrid)                               :: lon_out
  real, dimension(nygrid)                               :: lat_out
  real, dimension(:,:,:), allocatable                   :: work, workout, concout
  real, dimension(:,:,:,:), allocatable                 :: conc  
  integer                                               :: status
  logical                                               :: preslevel

  ! initialize
  ! ----------

  ! don't pass globals to subroutines
  lon_out = gbl_lon
  lat_out = gbl_lat
  nx_out = nxgrid
  ny_out = nygrid

  ! read netcdf file
  ! ----------------

  write(logid,*) 'Reading file: '//trim(filename)
  preslevel=.true.    !CGZ: defaults for noaa interpolated sfc fields
  midpoints = .false.

  ! read dimensions
  call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
  call check( nf90_inq_dimid(ncid,lonname,xid) )
  call check( nf90_inq_dimid(ncid,latname,yid) )

  !The data may be on pressure or z-levels, check this (info lost when using "check")
  status = nf90_inq_dimid(ncid,prsname,zid)
  if (status.eq.-46) then
      !Error code -46 is given when the dimension name does not exist, try if level rather than pressure is available.
      print*, 'Pressure not available, test if initial concentration on z-level.'
      status = nf90_inq_dimid(ncid,levname,zid)
      print*, status, trim(filename), levname

      call check( nf90_inq_dimid(ncid,levname,zid) )
      preslevel=.false.
      midpoints=.true. !DEFAULT FOR FLEXPART CTM NUDGING FIELDS
      print*, 'Read z-lev dim'
  else
      call check( nf90_inq_dimid(ncid,prsname,zid) )
  endif

  !Time does not have to be available, check this here (info lost when using "check")
  status=nf90_inq_dimid(ncid,timename,tid)
  if (status.eq.-46) then
    !No time variable, assume it's a mean value
    nt=1
  else
    call check( nf90_inq_dimid(ncid,timename,tid) ) !Make sure to write error if it was not -46
    call check( nf90_inquire_dimension(ncid,tid,len=nt) ) !Get size of time dimension
  endif

  call check( nf90_inquire_dimension(ncid,xid,len=nx) )
 ! print*, 'Read xid dim'
  call check( nf90_inquire_dimension(ncid,yid,len=ny) )
 ! print*, 'Read yid dim'
  call check( nf90_inquire_dimension(ncid,zid,len=nz) )
 ! print*, 'Read zid dim'

  ! mean so time dimension should be 1 
  if( nt.ne.1 ) then
     write(logid,*) 'ERROR get_initfpctm: time dimension should have length of 1 in '//trim(filename)
     stop
  endif

  allocate( lon(nx) )
  allocate( lat(ny) )
  allocate( prs(nz) )
  allocate( conc(nx,ny,nz,nt) )
  allocate( work(nx,ny,1) )
  allocate( workout(nxgrid,nygrid,1) ) 
  allocate( concout(nxgrid,nygrid,nz) )
  allocate( geoh(nz) )

  ! read variables
  call check( nf90_inq_varid(ncid,lonname,xid) )
  call check( nf90_get_var(ncid,xid,lon) )
  call check( nf90_inq_varid(ncid,latname,yid) )
  call check( nf90_get_var(ncid,yid,lat) )
  if(preslevel)then
    call check( nf90_inq_varid(ncid,prsname,zid) )
    call check( nf90_get_var(ncid,zid,prs) )
  else
    call check( nf90_inq_varid(ncid,levname,zid) )
    call check( nf90_get_var(ncid,zid,geoh) ) !Save height as geopotential height
  endif

  call check( nf90_inq_varid(ncid,trim(varname),varid) )
  call check( nf90_get_var(ncid,varid,conc(:,:,:,:)) )

  call check( nf90_close(ncid) )

  ! calculate geopotential height for each layer
  if(preslevel)then
    do kz = 1, nz
      geoh(kz) = (gasc*ts/gc/mmol)*log(psurf/prs(kz))
    enddo
  else
      !print*, 'CGZ, print debug, height lev:', geoh(:)
  endif


 ! print*, 'get_initfpctm: nx, ny, nz = ',nx,ny,nz
 ! print*, 'get_initfpctm: lon = ',lon
 ! print*, 'get_initfpctm: lat = ',lat


  ! interpolate to flexpart grid
  ! ----------------------------
  
  ! interpolate horizontally
  do kz = 1, nz
    work(:,:,1) = conc(:,:,kz,1)
    ! gbl_lon and gbl_lat are lon and lat of global flexpart domain
    call convertgrid(midpoints, nx, ny, lon, lat, work, nx_out, ny_out, lon_out, lat_out, workout)
    concout(:,:,kz) = workout(:,:,1)
  end do

  ! interpolate vertically
  call interpz(nz, geoh, concout, concini)
   !write(logid,*)'CGZ Writing grid to file:', trim(filename)//'_to_concini.txt'
   !write(logid,*)'CGZ grid dimension:', nxgrid, nygrid, nzgrid
   !write(logid,*)'CGZ min value grid:', MINVAL(concout(:,:,1)), MINVAL(concout(:,:,:))
   !open(100,file=trim(filename)//'_to_concini.txt',action='write',status='replace') 
   !do kz = 1, nzgrid
   !  do jy = 1, nygrid
   !    do ix = 1, nxgrid
   !    write(100,*) ix,jy,kz, concini(ix,jy,kz)
   !    end do
   !  end do
   !end do
   !close(100)

  deallocate( lon )
  deallocate( lat )
  deallocate( prs )
  deallocate( conc )
  deallocate( work )
  deallocate( workout )
  deallocate( concout )
  deallocate( geoh )

end subroutine get_initfpctm


