!---------------------------------------------------------------------------------------
! FLEXINVERT: read_obs
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> read_obs
!!
!! Purpose:    Read observations from files and write to obsread.txt
!!
!---------------------------------------------------------------------------------------

subroutine read_obs(config, files)

  use mod_settings
  use mod_var
  use mod_dates
  use mod_strings

  implicit none

  type (config_t), intent(in)                            :: config
  type (files_t),  intent(in)                            :: files
  character(len=max_path_len), dimension(:), allocatable :: filelist
  character(len=4)                                       :: recs
  character(len=200)                                     :: header
  integer                                                :: ierr
  integer                                                :: cnt
  integer                                                :: nf, nfiles
  integer                                                :: yyyymmdd, hhmiss
  real(kind=8)                                           :: jdate, avetime
  real                                                   :: conc, err, measerr
  integer                                                :: num, reclen

  ! list observation files
  ! ----------------------

  call system('ls '//trim(files%path_obs)//'*'//trim(files%suffix)//' | wc -l > '//trim(files%path_output)//'obsfiles.txt')
  call system('ls '//trim(files%path_obs)//' | grep '//trim(files%suffix)//' >> '//trim(files%path_output)//'obsfiles.txt')

  open(100,file=trim(files%path_output)//'obsfiles.txt',action='read',status='old',iostat=ierr)
  if ( ierr.ne. 0 ) then
    write(logid,*) 'ERROR: cannot open obsfiles.txt'
    stop
  endif
  read(100,*,iostat=ierr) nfiles
  allocate ( filelist(nfiles), stat = ierr )
  do nf = 1, nfiles
    read(100,fmt='(A)',iostat=ierr) filelist(nf)
    if (ierr.ne.0) exit
  end do
  close(100)

  ! loop over files
  ! ---------------

  ! output file for all observations
  open(200,file=trim(files%path_output)//'obsread.txt',action='write',status='old',iostat=ierr)
  if (ierr.ne.0) then
    write(logid,*) 'ERROR: cannot open obsread.txt'
    stop
  endif
  write(200,*) 'rec yyyymmdd hhmmss juldate avetime conc error nvalue'

  cnt = 0
  do nf = 1, nfiles

    ! check this file belongs to receptor in reclist
    ! added option for 4 character length names
    if (filelist(nf)(4:4).eq.'_' )then
      recs = filelist(nf)(1:3)
      reclen=3
    else
      recs = filelist(nf)(1:4)
      reclen=4
    endif
    if ( .not.( any(recname(:)(1:4).eq.to_upper(recs)) ) .and. .not.( any(recname(:)(1:3).eq.to_upper(recs)) )) go to 10

    ! open input file
    open(100,file=trim(files%path_obs)//trim(filelist(nf)),action='read',status='old',iostat=ierr)
    if ( ierr.ne.0 ) then
      write(logid,*) 'WARNING: cannot open: '//trim(files%path_obs)//trim(filelist(nf))
      go to 10
    endif 
    write(logid,*) 'Reading file: '//trim(files%path_obs)//trim(filelist(nf))

    ! read header
    read (100, fmt='(A)', iostat=ierr) header

    ! read data
    read_loop: do 
      ! for backwards compatibility use flag to indicate if column "avetime" was included in obs file
      if ( config%average_fp ) then
        ! if avetime included
        if (reclen.eq.4) then
            read(100,fmt='(A4,1X,I8,1X,I6,1X,F14.6,1X,F10.6,1X,F10.4,1X,F10.4,1X,I4)',iostat=ierr) &
                recs, yyyymmdd, hhmiss, jdate, avetime,  conc, err, num
        elseif (reclen.eq.3) then
            read(100,fmt='(A3,1X,I8,1X,I6,1X,F14.6,1X,F10.6,1X,F10.4,1X,F10.4,1X,I4)',iostat=ierr) &
                recs, yyyymmdd, hhmiss, jdate, avetime,  conc, err, num
        endif
      else
        ! if avetime not included
        avetime = 0d0
        if (reclen.eq.4) then
            read(100,fmt='(A4,1X,I8,1X,I6,1X,F14.6,1X,F10.4,1X,F10.4,1X,I4)',iostat=ierr) &
                recs, yyyymmdd, hhmiss, jdate, conc, err, num
        elseif (reclen.eq.3) then
            read(100,fmt='(A3,1X,I8,1X,I6,1X,F14.6,1X,F10.4,1X,F10.4,1X,I4)',iostat=ierr) &
                recs, yyyymmdd, hhmiss, jdate, conc, err, num
        endif
      endif
      ! round jdate
!      print*, 'read_obs: before roundingjdate = ',jdate
      jdate = dnint(jdate*1.e5)      
      jdate = jdate/1.e5
!      print*, 'read_obs: after rounding jdate = ',jdate
      if ( ierr.ne.0 ) exit read_loop
      if ( jdate.ge.(juldatef+1d0) ) exit read_loop
      if ( jdate.lt.juldatei ) cycle read_loop
      if ( conc.le.-999. ) cycle read_loop
      measerr = max(config%measerr, err)
      cnt = cnt + 1
      ! write to output file
      write(200,fmt='(A4,1X,I8,1X,I6,1X,F14.6,1X,F10.6,1X,F10.4,1X,F10.4)') &
              recs, yyyymmdd, hhmiss, jdate, avetime,conc, measerr
    end do read_loop

    ! close input file
    close(100)

10  continue

  end do

  ! close output file
  close(200)

  ! count observations
  nobs = nobs + cnt

end subroutine read_obs


